# # Ugly CSV generator
# [![Pypi project](https://badge.fury.io/py/ugly-csv-generator.svg)](https://badge.fury.io/py/ugly-csv-generator)
# [![Pypi total project downloads](https://pepy.tech/badge/ugly-csv-generator)](https://pepy.tech/projects/ugly-csv-generator)
# [![LICENSE](https://img.shields.io/pypi/l/ugly-csv-generator)](https://github.com/LucaCappelletti94/ugly-csv-generator/blob/main/LICENSE)
# [![Python version](https://img.shields.io/pypi/pyversions/ugly-csv-generator)](https://img.shields.io/pypi/pyversions/ugly-csv-generator)
# [![Github Actions](https://github.com/LucaCappelletti94/ugly_csv_generator/actions/workflows/python.yml/badge.svg)](https://github.com/LucaCappelletti94/ugly_csv_generator/actions/)
# [![Codacy Badge](https://app.codacy.com/project/badge/Grade/e6fe64db1c9042bbaa4c0a20bde585dc)](https://app.codacy.com/gh/LucaCappelletti94/ugly_csv_generator/dashboard?utm_source=gh&utm_medium=referral&utm_content=&utm_campaign=Badge_grade)
# 
# Python package to automatically uglify CSVs. Why? To improve the testing capabilities of pipelines that must be able to support strongly malformed input data.
# 
# All the malformation automated here are non-destructive, meaning they introduce confusion in the data but do not mangle or destroy information.
# 
# **The inspiration for the automated malformation are all from real-life CSVs (sigh)**
# 
# Humans will always surprise us with the ever-new malformed input data, but hey, we can try to best ruining the test CSVs!
# 
# ## How do I install this package?
# As usual, just download it using pip:
# 
# ```shell
# pip install ugly_csv_generator

# 
# ## Usage example
# 
# To ruin a CSV you can use the following snippet. In the following example we use a [random_csv_generator](https://github.com/LucaCappelletti94/random_csv_generator) to generate a random "healthy" csv.
# 
def test_line_27():
    from random_csv_generator import random_csv
    from ugly_csv_generator import uglify
    
    csv = random_csv(5) # CSV with 5 lines
    csv = csv[csv.columns[:3]] # We will use only the first 3 columns for this example
    ugly = uglify(csv)

# 
# The initial CSV will look something like:
# 
# | region    | province  | surname  |
# |-----------|-----------|----------|
# | Calabria  | Catanzaro | Rossi    |
# | Sicilia   | Ragusa    | Pinna    |
# | Lombardia | Varese    | Sbrana   |
# | Lazio     | Roma      | Mair     |
# | Sicilia   | Messina   | Ferrari  |
# 
# The result uglified CSV will look something like this:
# 
# |     | 1                                     | 2                   | 3        | 4        | 5                                      | 6    |
# |-----|---------------------------------------|---------------------|----------|----------|----------------------------------------|------|
# | 0   | ////                                  | #RIF!               | #RIF!    | 0        | ....                                   | 0    |
# | 1   | "('surname',)('.',)(0,)"              | region              | province | surname  | "('province',)('_',)(1,)"              |      |
# | 2   | ////////                              | region              | "province                                   " | "surname                                   " | 0                                      | 0    |
# | 3   | ///////                               | "region                                         " | "province                                   " | "surname                                     " | #RIF!                                   | #RIF!     |
# | 4   |                                       | Calabria            | "Catanzaro                                   " | "Rossi                                     " | 0                                      | -------- |
# | 5   | "                                     " | Sicilia            | Ragusa   | "Pinna                                     " | "                                            " |          |
# | 6   | -------                               |                     | #RIF!    | #RIF!    | 0                                      | "                                        " |
# | 7   | /////////                             | "Lombardia                                      " | "Varese                                     " | Sbrana                                  | ///////////                             |          |
# | 8   | ---------                             | "Lazio                                         " | "Roma                                       " | "Mair                                       " |                                        |          |
# | 9   | --------                              | 0                   | /////    | ---      | 0                                      | ///// |
# | 10  | #RIF!                                 | "Sicilia                                     " | Messina  | "Ferrari                                     " | 0                                      |          |
# | 11  | 0                                     |                     | -----    | "                                             " | --------                                | 0    |
# 
# ## Available uglifications
# Let's take a look at the available uglifications! All of these options are available as keyword arguments in the `uglify` function.
# 
# We start by taking a look at the same example from before, but now we expand all of the available options:
# 
def test_line_68():
    from random_csv_generator import random_csv
    from ugly_csv_generator import uglify
    
    csv = random_csv(5) # CSV with 5 lines
    csv = csv[csv.columns[:3]] # We will use only the first 3 columns for this example
    
    ugly = uglify(
        csv,
        empty_columns = True,
        empty_rows = True,
        duplicate_schema = True,
        empty_padding = True,
        nan_like_artefacts = True,
        replace_zeros = True,
        replace_ones = True,
        satellite_artefacts = False,
        random_spaces = True,
        include_unicode = True,
        verbose = True,
        seed = 42,
    )

# 
# Let's break down all of the available options with adequate examples. In all cases, we will use the following CSV as a starting point,
# obtained from the `random_csv_generator` package:
# 
def test_line_95():
    from random_csv_generator import random_csv
    
    csv = random_csv(5) # CSV with 5 lines
    csv = csv[csv.columns[:3]] # We will use only the first 3 columns for this example

# 
# The initial CSV will look something like:
# 
# |   | region  | province   | surname |
# |---|---------|------------|---------|
# | 0 | Veneto  | Vicenza    | Sacco   |
# | 1 | Abruzzo | L' Aquila  | Sala    |
# | 2 | Sicilia | Messina    | Sanna   |
# | 3 | Marche  | Ancona     | Gallo   |
# | 4 | Lazio   | Frosinone  | Gallo   |
# 
# ### Empty columns
# In the following example we will solely add empty columns to the CSV. This phenomenon is common when the data-entry person leaves empty columns in the middle of the table.
# 
def test_line_115():
    from random_csv_generator import random_csv
    from ugly_csv_generator import uglify
    
    csv = random_csv(5) # CSV with 5 lines
    csv = csv[csv.columns[:3]] # We will use only the first 3 columns for this example
    ugly = uglify(
        csv,
        empty_columns = True,
        empty_rows = False,
        duplicate_schema = False,
        empty_padding = False,
        nan_like_artefacts = False,
        satellite_artefacts = False,
        random_spaces = False,
        seed = 424,
    )

# 
# The result will look something like:
# 
# |   | region_2 | region_0 1 | region  | region_0 | province   | surname |
# |---|----------|------------|---------|----------|------------|---------|
# | 0 |          |            | Veneto  |          | Vicenza    | Sacco   |
# | 1 |          |            | Abruzzo |          | L Aquila   | Sala    |
# | 2 |          |            | Sicilia |          | Messina    | Sanna   |
# | 3 |          |            | Marche  |          | Ancona     | Gallo   |
# | 4 |          |            | Lazio   |          | Frosinone  | Gallo   |
# 
# ### Empty rows
# In the following example we will solely add empty rows to the CSV. This phenomenon is common when the data-entry person leaves empty rows in the middle of the table.
# 
def test_line_147():
    from random_csv_generator import random_csv
    from ugly_csv_generator import uglify
    
    csv = random_csv(5) # CSV with 5 lines
    csv = csv[csv.columns[:3]] # We will use only the first 3 columns for this example
    ugly = uglify(
        csv,
        empty_columns = False,
        empty_rows = True,
        duplicate_schema = False,
        empty_padding = False,
        nan_like_artefacts = False,
        satellite_artefacts = False,
        random_spaces = False,
        seed = 424,
    )

# 
# The result will look something like:
# 
# |   | region  | province   | surname |
# |---|---------|------------|---------|
# | 0 | Veneto  | Vicenza    | Sacco   |
# | 1 | Abruzzo | L Aquila   | Sala    |
# | 2 | Sicilia | Messina    | Sanna   |
# | 3 |         |            |         |
# | 4 | Marche  | Ancona     | Gallo   |
# | 5 | Lazio   | Frosinone  | Gallo   |
# | 6 |         |            |         |
# 
# ### Duplicate schema
# In the following example we will solely duplicate the schema of the CSV. This phenomenon is common when the data-entry person copies the header of the table multiple times, or several CSVs are concatenated together without removing the header.
# 
def test_line_181():
    from random_csv_generator import random_csv
    from ugly_csv_generator import uglify
    
    csv = random_csv(5) # CSV with 5 lines
    csv = csv[csv.columns[:3]] # We will use only the first 3 columns for this example
    ugly = uglify(
        csv,
        empty_columns = False,
        empty_rows = False,
        duplicate_schema = True,
        empty_padding = False,
        nan_like_artefacts = False,
        satellite_artefacts = False,
        random_spaces = False,
        seed = 424,
    )

# 
# The result will look something like:
# 
# |   | region  | province   | surname |
# |---|---------|------------|---------|
# | 0 | Veneto  | Vicenza    | Sacco   |
# | 1 | Abruzzo | L Aquila   | Sala    |
# | 2 | Sicilia | Messina    | Sanna   |
# | 3 | region  | province   | surname |
# | 4 | Marche  | Ancona     | Gallo   |
# | 5 | Lazio   | Frosinone  | Gallo   |
# | 6 | region  | province   | surname |
# 
# ### Empty padding
# In the following example we will solely add empty padding to the CSV. Padding in this context means adding empty cells around the CSV, represing when the data-entry person started the table somewhere in the middle of a sheet document.
# 
def test_line_215():
    from random_csv_generator import random_csv
    from ugly_csv_generator import uglify
    
    csv = random_csv(5) # CSV with 5 lines
    csv = csv[csv.columns[:3]] # We will use only the first 3 columns for this example
    ugly = uglify(
        csv,
        empty_columns = False,
        empty_rows = False,
        duplicate_schema = False,
        empty_padding = True,
        nan_like_artefacts = False,
        satellite_artefacts = False,
        random_spaces = False,
        seed = 424,
    )

# 
# The result will look something like:
# 
# |   |   0 | 1       | 2        | 3       | 4  | 5  |
# |---|-----|---------|----------|---------|----|----|
# | 0 |     | region  | province | surname |    |    |
# | 1 |     | Veneto  | Vicenza  | Sacco   |    |    |
# | 2 |     | Abruzzo | L Aquila | Sala    |    |    |
# | 3 |     | Sicilia | Messina  | Sanna   |    |    |
# | 4 |     | Marche  | Ancona   | Gallo   |    |    |
# | 5 |     | Lazio   | Frosinone| Gallo   |    |    |
# | 6 |     |         |          |         |    |    |
# | 7 |     |         |          |         |    |    |
# | 8 |     |         |          |         |    |    |
# | 9 |     |         |          |         |    |    |
# | 10|     |         |          |         |    |    |
# | 11|     |         |          |         |    |    |
# 
# ### NaN-like artefacts
# In the following example we will solely add NaN-like artefacts to the CSV. This phenomenon is common when the data-entry person follows some custom notation, which may be their own or office standard, to represent missing values. In some cases, this may be a string like "N/A", "NaN", "NULL", or even (one or more) "-", "\n", or "\t". Since the objective of this package is to not destroy information, it will solely replace NaN values with NaN-like artefacts.
# 
# In the example we considered earlier, we do not have any NaN values, so we will add some to the CSV by also enabling the `empty_rows` option.
# 
def test_line_256():
    from random_csv_generator import random_csv
    from ugly_csv_generator import uglify
    
    csv = random_csv(5) # CSV with 5 lines
    csv = csv[csv.columns[:3]] # We will use only the first 3 columns for this example
    ugly = uglify(
        csv,
        empty_columns = False,
        empty_rows = True,
        duplicate_schema = False,
        empty_padding = False,
        nan_like_artefacts = True,
        satellite_artefacts = False,
        random_spaces = False,
        seed = 424,
    )

# 
# The result will look something like:
# 
# |   | region  | province   | surname |
# |---|---------|------------|---------|
# | 0 | Veneto  | Vicenza    | Sacco   |
# | 1 | Abruzzo | L Aquila   | Sala    |
# | 2 | Sicilia | Messina    | Sanna   |
# | 3 | " "     | ...        | ----    |
# | 4 | Marche  | Ancona     | Gallo   |
# | 5 | Lazio   | Frosinone  | Gallo   |
# | 6 |         | "          | ------- |
# 
# 
# #### Unicode variant
# The NaN-like artefacts can also be applied with unicode characters. This is useful to test the robustness of the CSV reader to unicode characters.
# 
def test_line_291():
    from random_csv_generator import random_csv
    from ugly_csv_generator import uglify
    
    csv = random_csv(5) # CSV with 5 lines
    csv = csv[csv.columns[:3]] # We will use only the first 3 columns for this example
    
    ugly = uglify(
        csv,
        empty_columns = False,
        empty_rows = True,
        duplicate_schema = False,
        empty_padding = False,
        nan_like_artefacts = True,
        satellite_artefacts = False,
        random_spaces = False,
        include_unicode = True,
        seed = 424,
    )

# 
# The result will look something like:
# 
# |    | region    | province   | surname   |
# |---:|:----------|:-----------|:----------|
# |  0 | Calabria  | Catanzaro  | Rossi     |
# |  1 | Sicilia   | Ragusa     | Pinna     |
# |  2 | Lombardia | Varese     | Sbrana    |
# |  3 | .         | ᴑ          | 0         |
# |  4 | Lazio     | Roma       | Mair      |
# |  5 | Sicilia   | Messina    | Ferrari   |
# |  6 | ₀         | ________   | ᪐         |
# 
# ### Replace zeros
# In the following example we will solely replace zeros with a custom value. In different places in the word and different offices, zeros may be represented in different ways. Characters for zero from different alphabets, or even different symbols, may be used to represent zero. Note that this latter functionality is only enabled if the `include_unicode` option is set to `True`.
# 
def test_line_327():
    from random_csv_generator import random_csv
    from ugly_csv_generator import uglify
    
    csv = random_csv(5) # CSV with 5 lines
    csv = csv[csv.columns[:3]] # We will use only the first 3 columns for this example
    
    # We add a column with zeros
    csv["zero"] = 0
    
    ugly = uglify(
        csv,
        empty_columns = False,
        empty_rows = False,
        duplicate_schema = False,
        empty_padding = False,
        nan_like_artefacts = False,
        satellite_artefacts = False,
        random_spaces = False,
        replace_zeros = True,
        include_unicode = False,
        seed = 424,
    )

# 
# The result will look something like:
# 
# |    | region    | province   | surname   | zero   |
# |---:|:----------|:-----------|:----------|:-------|
# |  0 | Calabria  | Catanzaro  | Rossi     | 0      |
# |  1 | Sicilia   | Ragusa     | Pinna     | o      |
# |  2 | Lombardia | Varese     | Sbrana    | 0      |
# |  3 | Lazio     | Roma       | Mair      | 0      |
# |  4 | Sicilia   | Messina    | Ferrari   | O      |
# 
# #### Unicode variant
# The replace zeros can also be applied with unicode characters. This is useful to test the robustness of the CSV reader to unicode characters.
# 
def test_line_365():
    from random_csv_generator import random_csv
    from ugly_csv_generator import uglify
    
    csv = random_csv(5) # CSV with 5 lines
    csv = csv[csv.columns[:3]] # We will use only the first 3 columns for this example
    
    # We add a column with zeros
    csv["zero"] = 0
    
    ugly = uglify(
        csv,
        empty_columns = False,
        empty_rows = False,
        duplicate_schema = False,
        empty_padding = False,
        nan_like_artefacts = False,
        satellite_artefacts = False,
        random_spaces = False,
        replace_zeros = True,
        include_unicode = True,
        seed = 424,
    )

# 
# The result will look something like:
# 
# |    | region    | province   | surname   | zero   |
# |---:|:----------|:-----------|:----------|:-------|
# |  0 | Calabria  | Catanzaro  | Rossi     | o      |
# |  1 | Sicilia   | Ragusa     | Pinna     | ᪐      |
# |  2 | Lombardia | Varese     | Sbrana    | ο      |
# |  3 | Lazio     | Roma       | Mair      | 𝟘      |
# |  4 | Sicilia   | Messina    | Ferrari   | ᥆      |
# 
# ### Replace ones
# In the following example we will solely replace ones with a custom value. In different places in the word and different offices, ones may be represented in different ways. Characters for one from different alphabets, or even different symbols, may be used to represent one. Note that this latter functionality is only enabled if the `include_unicode` option is set to `True`.
# 
def test_line_403():
    from random_csv_generator import random_csv
    from ugly_csv_generator import uglify
    
    csv = random_csv(5) # CSV with 5 lines
    csv = csv[csv.columns[:3]] # We will use only the first 3 columns for this example
    
    # We add a column with ones
    csv["one"] = 1
    
    ugly = uglify(
        csv,
        empty_columns = False,
        empty_rows = False,
        duplicate_schema = False,
        empty_padding = False,
        nan_like_artefacts = False,
        satellite_artefacts = False,
        random_spaces = False,
        replace_ones = True,
        include_unicode = False,
        seed = 424,
    )

# 
# The result will look something like:
# 
# |    | region    | province   | surname   | one   |
# |---:|:----------|:-----------|:----------|:------|
# |  0 | Calabria  | Catanzaro  | Rossi     | 1     |
# |  1 | Sicilia   | Ragusa     | Pinna     | l     |
# |  2 | Lombardia | Varese     | Sbrana    | 1     |
# |  3 | Lazio     | Roma       | Mair      | 1     |
# |  4 | Sicilia   | Messina    | Ferrari   | I     |
# 
# #### Unicode variant
# The replace ones can also be applied with unicode characters. This is useful to test the robustness of the CSV reader to unicode characters.
# 
def test_line_441():
    from random_csv_generator import random_csv
    from ugly_csv_generator import uglify
    
    csv = random_csv(5) # CSV with 5 lines
    csv = csv[csv.columns[:3]] # We will use only the first 3 columns for this example
    
    # We add a column with ones
    csv["one"] = 1
    
    ugly = uglify(
        csv,
        empty_columns = False,
        empty_rows = False,
        duplicate_schema = False,
        empty_padding = False,
        nan_like_artefacts = False,
        satellite_artefacts = False,
        random_spaces = False,
        replace_ones = True,
        include_unicode = True,
        seed = 424,
    )

# 
# The result will look something like:
# 
# |    | region    | province   | surname   | one   |
# |---:|:----------|:-----------|:----------|:------|
# |  0 | Calabria  | Catanzaro  | Rossi     | ¹     |
# |  1 | Sicilia   | Ragusa     | Pinna     | ₁     |
# |  2 | Lombardia | Varese     | Sbrana    | l     |
# |  3 | Lazio     | Roma       | Mair      | 1     |
# |  4 | Sicilia   | Messina    | Ferrari   | ⓵     |
# 
# 
# ### Satellite artefacts
# In the following example we will solely add satellite artefacts to the CSV. A satellite artefact is likely the quirkiest and most annoying artefact to deal with. It represents the situation where the data-entry person adds some notes on the side of the table. A real-world example of this which I have encountered is when the data-entry person adds the office lunch order on the side of the table and forgets to remove it.
# 
# The package offers a few satellite artefacts encountered in the wild.
# 
def test_line_482():
    from random_csv_generator import random_csv
    from ugly_csv_generator import uglify
    
    csv = random_csv(5) # CSV with 5 lines
    csv = csv[csv.columns[:3]] # We will use only the first 3 columns for this example
    ugly = uglify(
        csv,
        empty_columns = False,
        empty_rows = True,
        duplicate_schema = False,
        empty_padding = False,
        nan_like_artefacts = False,
        satellite_artefacts = True,
        random_spaces = False,
        seed = 424,
    )

# 
# The result will look something like:
# 
# |    | 0       | 1         | 2                | 3       | 4  |
# |----|---------|-----------|------------------|---------|----|
# | 0  |         |           |                  | random  |    |
# | 1  |         |           | random           |         |    |
# | 2  |         | caso      |                  |         |    |
# | 3  | region  | province  | surname          |         |    |
# | 4  | Veneto  | Vicenza   | Sacco            |         |    |
# | 5  | Abruzzo | L Aquila  | Sala             |         |    |
# | 6  | Sicilia | Messina   | Sanna            |         |    |
# | 7  | Marche  | Ancona    | Gallo            |         |    |
# | 8  | Lazio   | Frosinone | Gallo            |         |    |
# | 9  |         |           |                  |         |    |
# | 10 |         |           |                  |         |    |
# | 11 |         |           |                  |         |    |
# | 12 |         |           |                  |         |    |
# | 13 |         |           |                  |         |    |
# | 14 |         |           |                  |         |    |
# | 15 | person  | food      |                  |         |    |
# | 16 | Jerry   | kebab     |                  |         |    |
# | 17 | Steven  | rice with paprika |          |         |    |
# | 18 | Vale    | pizza mit ananas |          |         |    |
# 
# ### Random spaces
# In the following example we will solely add random spaces around the values in the CSV. This phenomenon is common when the data-entry person is not careful with the spaces around the values in the table and adds some random spaces, for instance to visually align the values.
# 
def test_line_528():
    from random_csv_generator import random_csv
    from ugly_csv_generator import uglify
    
    csv = random_csv(5) # CSV with 5 lines
    csv = csv[csv.columns[:3]] # We will use only the first 3 columns for this example
    ugly = uglify(
        csv,
        empty_columns = False,
        empty_rows = False,
        duplicate_schema = False,
        empty_padding = False,
        nan_like_artefacts = False,
        satellite_artefacts = False,
        random_spaces = True,
        seed = 424,
    )

# 
# The result will look something like:
# 
# |   | region               | province         | surname         |
# |---|----------------------|------------------|-----------------|
# | 0 | "    Veneto          " | "  Vicenza      " | " Sacco        " |
# | 1 | " Abruzzo            " | " L Aquila      " | " Sala         " |
# | 2 | " Sicilia            " | " Messina       " | " Sanna        " |
# | 3 | " Marche             " | " Ancona        " | " Gallo        " |
# | 4 | " Lazio              " | " Frosinone     " | " Gallo        " |
# 
# 
# #### Unicode variant
# The random spaces uglification can also be applied with unicode characters. This is useful to test the robustness of the CSV reader to unicode characters.
# 
def test_line_561():
    from random_csv_generator import random_csv
    from ugly_csv_generator import uglify
    
    csv = random_csv(5) # CSV with 5 lines
    csv = csv[csv.columns[:3]] # We will use only the first 3 columns for this example
    ugly = uglify(
        csv,
        empty_columns = False,
        empty_rows = False,
        duplicate_schema = False,
        empty_padding = False,
        nan_like_artefacts = False,
        satellite_artefacts = False,
        random_spaces = True,
        include_unicode = True,
        seed = 424,
    )

# 
# Due to limitations of the markdown rendering, we cannot show the result here. You can run the code snippet to see the result. It's just that damn cursed!
# 
# ## Contributing
# You have encountered a new type of uglification that you would like to add to the package? You have a suggestion for a new feature or improvement? You have found a bug? Open an issue or a pull request, I will be happy to help you!
# 
# ## License
# This project is licensed under the MIT License - see the [LICENSE](https://github.com/LucaCappelletti94/ugly_csv_generator/blob/master/LICENSE) file for details.



"""Submodule for adding a row to a DataFrame."""

from typing import List
from random import Random
import pandas as pd
from ugly_csv_generator.utils.add_chunk import add_chunk


def add_row(csv: pd.DataFrame, values: List, iloc: int = None, state: Random = None):
    """Add a row to a DataFrame.

    Parameters
    --------------------------
    csv: pd.DataFrame,
        The DataFrame to add the row to.
    values: List,
        The values to add.
    iloc: int = None,
        The index location where to add the row.
    state: Random = None,
        The random state to use for the generation.
    """
    if iloc is None:
        iloc = state.randint(0, len(csv))
    return add_chunk(
        csv, pd.DataFrame(dict(zip(csv.columns, values)), index=[iloc]), iloc
    )



"""Add a chunk to a DataFrame at a given index."""

import pandas as pd
import numpy as np


def add_chunk(csv: pd.DataFrame, chunk: pd.DataFrame, iloc: int):
    """Add a chunk to a DataFrame at a given index.

    Parameters
    --------------------------
    csv: pd.DataFrame,
        The DataFrame to add the chunk to.
    chunk: pd.DataFrame,
        The chunk to add.
    iloc: int,
        The index location where to add the chunk.
    """
    if chunk.shape[1] < csv.shape[1]:
        old_chunk = chunk
        chunk = pd.DataFrame(
            np.nan, columns=csv.columns, index=old_chunk.index, dtype="object"
        )
        chunk.values[: old_chunk.shape[0], : old_chunk.shape[1]] = old_chunk.values
    return pd.concat([csv.iloc[:iloc], chunk, csv.iloc[iloc:]]).reset_index(drop=True)



"""Submodule providing an utility to add random empty columns to a CSV."""

from random import Random
import pandas as pd
import numpy as np
from tqdm.auto import trange


def add_empty_columns(csv: pd.DataFrame, state: Random, verbose: bool) -> pd.DataFrame:
    """Add random empty columns to the provided DataFrame.

    Parameters
    --------------------------
    csv: pd.DataFrame,
        The DataFrame to add the empty columns to.
    state: Random,
        The random state to use for the generation.
    verbose: bool,
        Whether to show the loading bars.
    """
    columns_number = state.randint(0, len(csv.columns))
    for number in trange(
        columns_number,
        desc="Adding empty columns",
        disable=not verbose or columns_number <= 1,
        leave=False,
        dynamic_ncols=True,
    ):
        loc = state.randint(0, len(csv.columns))
        column = (state.choice(csv.columns),)
        separator = (state.choice(("", " ", ".", "_", "-")),)
        number = (number,)
        csv.insert(
            loc=loc,
            column=f"{column}{separator}{number}",
            value=np.nan,
        )
    return csv



"""Test the uglify function."""

import random
import os
from random_csv_generator import random_csv
import pandas as pd
from tqdm.auto import trange
from ugly_csv_generator import uglify


def test_uglify():
    """Test the uglify function."""
    state = random.Random(1234567890)
    for i in trange(100, leave=False):
        for include_unicode in [True, False]:
            csv = random_csv(
                number_of_rows=state.randint(1, 100), random_state=(i + 1) * 34567890
            )
            
            for j in trange(3, leave=False):
                csv_inner = csv[csv.columns[(j + 1) * 2 :]]
                ugly1 = uglify(csv_inner, seed=i, satellite_artefacts=True, replace_ones=True, replace_zeros=True, include_unicode=include_unicode)
                ugly2 = uglify(csv_inner, seed=i, satellite_artefacts=True, replace_ones=True, replace_zeros=True, include_unicode=include_unicode)

                pd.testing.assert_frame_equal(ugly1, ugly2)


def test_loading_csv_from_path():
    """Test the loading of a csv from a path."""
    state = random.Random(1234567890)
    for i in trange(100, leave=False):
        for include_unicode in [True, False]:
            csv = random_csv(
                number_of_rows=state.randint(1, 100), random_state=(i + 1) * 34567890
            )

            for j in trange(3, leave=False):
                csv_inner = csv[csv.columns[(j + 1) * 2 :]]
                csv_inner.to_csv("test.csv", index=False)
                ugly1 = uglify("test.csv", seed=i, satellite_artefacts=True, replace_ones=True, replace_zeros=True, include_unicode=include_unicode)
                ugly2 = uglify("test.csv", seed=i, satellite_artefacts=True, replace_ones=True, replace_zeros=True, include_unicode=include_unicode)

                pd.testing.assert_frame_equal(ugly1, ugly2)

    if os.path.exists("test.csv"):
        os.remove("test.csv")



"""Test to verify that all characters lists used throughout the package are unique."""

from ugly_csv_generator.utils.replace_zeros import (
    ZERO_LOOKING_CHARACTERS,
    UNICODE_ZERO_LOOKING_CHARACTERS,
)
from ugly_csv_generator.utils.add_nan_like_artefacts import (
    NAN_LIKE_ARTIFACTS,
    UNICODE_NAN_LIKE_ARTIFACTS,
)
from ugly_csv_generator.utils.add_random_spaces import (
    UNICODE_SPACES,
    SPACES,
)
from ugly_csv_generator.utils.replace_ones import (
    ONE_LOOKING_CHARACTERS,
    UNICODE_ONE_LOOKING_CHARACTERS,
)


def test_uniqueness():
    """Test to verify that all characters lists used throughout the package are unique."""
    assert len(ZERO_LOOKING_CHARACTERS) == len(
        set(ZERO_LOOKING_CHARACTERS)
    ), "The ZERO_LOOKING_CHARACTERS list is not unique."
    assert len(UNICODE_ZERO_LOOKING_CHARACTERS) == len(
        set(UNICODE_ZERO_LOOKING_CHARACTERS)
    ), "The UNICODE_ZERO_LOOKING_CHARACTERS list is not unique."
    assert len(NAN_LIKE_ARTIFACTS) == len(
        set(NAN_LIKE_ARTIFACTS)
    ), "The NAN_LIKE_ARTIFACTS list is not unique."
    assert len(UNICODE_NAN_LIKE_ARTIFACTS) == len(
        set(UNICODE_NAN_LIKE_ARTIFACTS)
    ), "The UNICODE_NAN_LIKE_ARTIFACTS list is not unique."
    assert len(UNICODE_SPACES) == len(
        set(UNICODE_SPACES)
    ), "The UNICODE_SPACES list is not unique."
    assert len(SPACES) == len(set(SPACES)), "The SPACES list is not unique."
    assert len(ONE_LOOKING_CHARACTERS) == len(
        set(ONE_LOOKING_CHARACTERS)
    ), "The ONE_LOOKING_CHARACTERS list is not unique."
    assert len(UNICODE_ONE_LOOKING_CHARACTERS) == len(
        set(UNICODE_ONE_LOOKING_CHARACTERS)
    ), "The UNICODE_ONE_LOOKING_CHARACTERS list is not unique."



"""Module providing the uglify function to uglify a CSV."""

from typing import Union
from random import Random
import pandas as pd
from ugly_csv_generator.utils import (
    add_empty_columns,
    add_empty_rows,
    add_duplicate_schema,
    add_empty_padding,
    add_nan_like_artefacts,
    add_random_spaces,
    add_satellites,
    replace_zeros as _replace_zeros,
    replace_ones as _replace_ones,
)


def uglify(
    csv: Union[pd.DataFrame, str],
    empty_columns: bool = True,
    empty_rows: bool = True,
    duplicate_schema: bool = True,
    empty_padding: bool = True,
    nan_like_artefacts: bool = True,
    replace_zeros: bool = False,
    replace_ones: bool = False,
    satellite_artefacts: bool = False,
    random_spaces: bool = True,
    include_unicode: bool = False,
    verbose: bool = True,
    seed: int = 42,
):
    """Return or saves to given path an uglified version of the given DataFrame.

    The procedure is non-destructive,
    meaning all infomation should remain retrievable
    using a valid reconstruction method.

    Parameters
    ---------------------------------
    csv: pd.DataFrame,
        The dataframe containing or path to the CSV to uglify.
    empty_columns: bool = True,
        Whether to introduce empty columns (with header).
    empty_rows: bool = True,
        Whether to introduce random blank rows.
    duplicate_schema: bool = True,
        Whether to duplicate the schema rows within the data.
    empty_padding: bool = True,
        Whether to introduce padding around the data.
    nan_like_artefacts: bool = True,
        Whether to introduce NaN-like artefacts.
        A NaN-like artefact is a value that looks like NaN
        but is not actually NaN, like "N/A" or "-".
    replace_zeros: bool = False,
        Whether to replace zeros with zero-looking characters,
        such as 'O' or 'o'.
    replace_ones: bool = False,
        Whether to replace ones with one-looking characters,
        such as 'I', 'i', or '|'.
    satellite_artefacts: bool = True,
        Whether to add satellite text around the central table.
        This is useful to simulate when you can expect for
        random scribbling around your CSV data.
        The satellites may be added on the top or bottom.
        It selects a random satellite from the available satellites
        collection in the package, encountered in the real world.
    random_spaces: bool = True,
        Whether to add random spaces to the file values.
    include_unicode: bool = False,
        Whether to include Unicode artefacts.
        This includes adding Unicode space-like artefacts or
        Unicode NaN-like artefacts.
    verbose: bool = True,
        Whether to show the loading bars.
    seed: int = 42
        The random seed to use to make reproducible the random aspects.

    Returns
    --------------------------------
    The uglified dataframe.
    """
    state = Random(seed)

    # If the CSV is a path, load it
    if isinstance(csv, str):
        csv = pd.read_csv(csv)

    if duplicate_schema:
        csv = add_duplicate_schema(csv, state, verbose=verbose)

    if replace_zeros:
        csv = _replace_zeros(csv, state, include_unicode=include_unicode)

    if replace_ones:
        csv = _replace_ones(csv, state, include_unicode=include_unicode)

    if random_spaces:
        csv = add_random_spaces(csv, state, include_unicode=include_unicode)

    if empty_columns:
        csv = add_empty_columns(csv.copy(), state, verbose=verbose)

    if empty_rows:
        csv = add_empty_rows(csv, state, verbose=verbose)

    if empty_padding:
        csv = add_empty_padding(csv, state)

    if satellite_artefacts:
        csv = add_satellites(csv, state)

    if nan_like_artefacts:
        csv = add_nan_like_artefacts(csv, state, include_unicode=include_unicode)

    return csv



"""Submodule providing utilities to add artefacts to a CSV."""

from ugly_csv_generator.utils.add_empty_columns import add_empty_columns
from ugly_csv_generator.utils.add_empty_rows import add_empty_rows
from ugly_csv_generator.utils.add_duplicate_schema import add_duplicate_schema
from ugly_csv_generator.utils.add_empty_padding import add_empty_padding
from ugly_csv_generator.utils.add_nan_like_artefacts import add_nan_like_artefacts
from ugly_csv_generator.utils.add_random_spaces import add_random_spaces
from ugly_csv_generator.utils.add_satellites import add_satellites
from ugly_csv_generator.utils.replace_zeros import replace_zeros
from ugly_csv_generator.utils.replace_ones import replace_ones

__all__ = [
    "add_empty_columns",
    "add_empty_rows",
    "add_duplicate_schema",
    "add_empty_padding",
    "add_nan_like_artefacts",
    "add_random_spaces",
    "add_satellites",
    "replace_zeros",
    "replace_ones",
]



"""Randomly add spaces, newlines, and carriage returns to a CSV file.

This submodule provides a function to add random spaces,
newlines, and carriage returns to a CSV file.
"""

from random import Random
import pandas as pd

SPACES = [" ", "\t", "\n", "\n\r"]

UNICODE_SPACES = [
    "\u00A0",  # Non-breaking space
    "\u1680",  # Ogham space mark
    "\u2000",  # En quad
    "\u2001",  # Em quad
    "\u2002",  # En space
    "\u2003",  # Em space
    "\u2004",  # Three-per-em space
    "\u2005",  # Four-per-em space
    "\u2006",  # Six-per-em space
    "\u2007",  # Figure space
    "\u2008",  # Punctuation space
    "\u2009",  # Thin space
    "\u200A",  # Hair space
    "\u200B",  # Zero-width space
    "\u200C",  # Zero-width non-joiner
    "\u200D",  # Zero-width joiner
    "\u202F",  # Narrow no-break space
    "\u205F",  # Medium mathematical space
    "\u3000",  # Ideographic space
    "\uFEFF",  # Zero-width non-breaking space
    "\u180E",  # Mongolian vowel separator
    "\u2060",  # Word joiner
    "\u2063",  # Invisible separator
    "\U000E0020",  # Tag space
    "\u001C",  # File separator
    "\u001D",  # Group separator
    "\u001E",  # Record separator
    "\u001F",  # Unit separator
]


def random_space(state: Random, include_unicode: bool):
    """Generate a random space-like artefact.

    Parameters
    --------------------------
    state: Random,
        The random state to use for the generation.
    include_unicode: bool,
        Whether to include Unicode space-like artefacts.

    Returns
    --------------------------
    A random space-like artefact.
    """
    k = state.randint(1, 10)
    return "".join(state.choices((UNICODE_SPACES + SPACES)  if include_unicode else SPACES, k=k))


def add_random_spaces(
    csv: pd.DataFrame, state: Random, include_unicode: bool
) -> pd.DataFrame:
    """Add random spaces, newlines, and carriage returns to the provided DataFrame.

    Parameters
    --------------------------
    csv: pd.DataFrame,
        The DataFrame to add the random spaces, newlines, and carriage returns to.
    state: Random,
        The random state to use for the generation.
    include_unicode: bool,
        Whether to include Unicode space-like artefacts.
    """
    return csv.map(
        lambda x: (
            f"{random_space(state, include_unicode)}{str(x).replace(' ', random_space(state, include_unicode))}{random_space(state, include_unicode)}"
            if not pd.isna(x)
            else x
        )
    )



"""Submodule providing a method to replace ones in a dataframe with one-looking characters."""

from random import Random
from typing import List
import pandas as pd
from ugly_csv_generator.utils.replace_character import replace_character

ONE_LOOKING_CHARACTERS: List[str] = ['I', 'l', '1']

UNICODE_ONE_LOOKING_CHARACTERS: List[str] = [
    '\uFF11',  # FULLWIDTH DIGIT ONE
    '\u2460',  # CIRCLED DIGIT ONE
    '\u24F5',  # DOUBLE CIRCLED DIGIT ONE
    '\u2776',  # DINGBAT NEGATIVE CIRCLED DIGIT ONE
    '\U0001D7D9',  # MATHEMATICAL DOUBLE-STRUCK DIGIT ONE
    '\U0001D7CF',  # MATHEMATICAL BOLD DIGIT ONE
    '\u2081',  # SUBSCRIPT ONE
    '\u00B9',  # SUPERSCRIPT ONE
    '\u0661',  # ARABIC-INDIC DIGIT ONE
    '\u06F1',  # EXTENDED ARABIC-INDIC DIGIT ONE (Persian)
    '\u07C1',  # NKO DIGIT ONE
    '\u0967',  # DEVANAGARI DIGIT ONE
    '\u09E7',  # BENGALI DIGIT ONE
    '\u0A67',  # GURMUKHI DIGIT ONE
    '\u0AE7',  # GUJARATI DIGIT ONE
    '\u0B67',  # ORIYA DIGIT ONE
    '\u0BE7',  # TAMIL DIGIT ONE
    '\u0C67',  # TELUGU DIGIT ONE
    '\u0CE7',  # KANNADA DIGIT ONE
    '\u0D67',  # MALAYALAM DIGIT ONE
    '\u0E51',  # THAI DIGIT ONE
    '\u0ED1',  # LAO DIGIT ONE
    '\u0F21',  # TIBETAN DIGIT ONE
    '\u1041',  # MYANMAR DIGIT ONE
    '\u17E1',  # KHMER DIGIT ONE
    '\u1811',  # MONGOLIAN DIGIT ONE
    '\u1947',  # LIMBU DIGIT ONE
    '\u19D1',  # NEW TAI LUE DIGIT ONE
    '\u1A81',  # TAI THAM HORA DIGIT ONE
    '\u1A91',  # TAI THAM THAM DIGIT ONE
    '\u1B51',  # BALINESE DIGIT ONE
    '\u1BB1',  # SUNDANESE DIGIT ONE
    '\u1C41',  # LEPCHA DIGIT ONE
    '\u1C51',  # OL CHIKI DIGIT ONE
]


def replace_ones(
    csv: pd.DataFrame, state: Random, include_unicode: bool
) -> pd.DataFrame:
    """Replace ones in a dataframe with one-looking characters.

    Parameters
    --------------------------
    csv: pd.DataFrame,
        The DataFrame to add the NaN-like artefacts to.
    state: Random,
        The random state to use for the generation.
    include_unicode: bool,
        Whether to include Unicode one-looking characters.
    """
    return replace_character(
        csv,
        state,
        ["1", 1, 1.0],
        (
            (UNICODE_ONE_LOOKING_CHARACTERS + ONE_LOOKING_CHARACTERS)
            if include_unicode
            else ONE_LOOKING_CHARACTERS
        ),
    )



"""Submodule providing a method to replace zeros in a dataframe with zero-looking characters."""

from random import Random
from typing import List
import pandas as pd
from ugly_csv_generator.utils.replace_character import replace_character

ZERO_LOOKING_CHARACTERS: List[str] = ['O', 'o', '0']

UNICODE_ZERO_LOOKING_CHARACTERS: List[str] = [
    '\uFF10',  # FULLWIDTH DIGIT ZERO
    '\u24EA',  # CIRCLED DIGIT ZERO
    '\u24FF',  # NEGATIVE CIRCLED DIGIT ZERO
    '\U0001D7D8',  # MATHEMATICAL DOUBLE-STRUCK DIGIT ZERO
    '\U0001D7CE',  # MATHEMATICAL BOLD DIGIT ZERO
    '\u2070',  # SUPERSCRIPT ZERO
    '\u2080',  # SUBSCRIPT ZERO
    '\u03BF',  # GREEK SMALL LETTER OMICRON
    '\u1D0F',  # LATIN LETTER SMALL CAPITAL O
    '\u1D11',  # LATIN SMALL LETTER SIDEWAYS O
    '\u1D13',  # LATIN SMALL LETTER TURNED O
    '\u1D52',  # MODIFIER LETTER SMALL O
    '\u041E',  # CYRILLIC CAPITAL LETTER O
    '\u043E',  # CYRILLIC SMALL LETTER O
    '\u0555',  # ARMENIAN CAPITAL LETTER VO
    '\u0585',  # ARMENIAN SMALL LETTER VO
    '\u0660',  # ARABIC-INDIC DIGIT ZERO
    '\u06F0',  # EXTENDED ARABIC-INDIC DIGIT ZERO (Persian)
    '\u07C0',  # NKO DIGIT ZERO
    '\u0E50',  # THAI DIGIT ZERO
    '\u0ED0',  # LAO DIGIT ZERO
    '\u0F20',  # TIBETAN DIGIT ZERO
    '\u1040',  # MYANMAR DIGIT ZERO
    '\u17E0',  # KHMER DIGIT ZERO
    '\u1810',  # MONGOLIAN DIGIT ZERO
    '\u1946',  # LIMBU DIGIT ZERO
    '\u19D0',  # NEW TAI LUE DIGIT ZERO
    '\u1A80',  # TAI THAM HORA DIGIT ZERO
    '\u1A90',  # TAI THAM THAM DIGIT ZERO
    '\u1B50',  # BALINESE DIGIT ZERO
    '\u1BB0',  # SUNDANESE DIGIT ZERO
    '\u1C40',  # LEPCHA DIGIT ZERO
    '\u1C50',  # OL CHIKI DIGIT ZERO
]


def replace_zeros(
    csv: pd.DataFrame, state: Random, include_unicode: bool
) -> pd.DataFrame:
    """Replace zeros in a dataframe with zero-looking characters.

    Parameters
    --------------------------
    csv: pd.DataFrame,
        The DataFrame to add the NaN-like artefacts to.
    state: Random,
        The random state to use for the generation.
    include_unicode: bool,
        Whether to include Unicode zero-looking characters.
    """
    return replace_character(
        csv,
        state,
        [0, 0.0, '0'],
        (UNICODE_ZERO_LOOKING_CHARACTERS + ZERO_LOOKING_CHARACTERS) if include_unicode else ZERO_LOOKING_CHARACTERS,
    )



"""Submodule providing a method to replace randomly in a dataframe given a random state a character with ones from a provided list."""

from random import Random
from typing import List
import pandas as pd


def replace_character(
    csv: pd.DataFrame, state: Random, character: List[str], characters_to_replace: List[str]
) -> pd.DataFrame:
    """Replace randomly in a dataframe given a random state a character with ones from a provided list.

    Parameters
    --------------------------
    csv: pd.DataFrame,
        The DataFrame to add the NaN-like artefacts to.
    state: Random,
        The random state to use for the generation.
    character: List[str],
        The character to replace.
        E.g. in the case of zero it will be ['0', 0, 0.0].
    characters_to_replace: List[str],
        The list of characters to replace the character with.
    """
    assert (
        characters_to_replace
    ), "The list of characters to replace the character with cannot be empty."
    return csv.map(
        lambda x: (state.choice(characters_to_replace) if x in character else x)
    )



"""Submodule for adding empty padding to the csv file."""

from random import Random
import pandas as pd
import numpy as np
from ugly_csv_generator.utils.add_row import add_row


def add_empty_padding(csv: pd.DataFrame, state: Random) -> pd.DataFrame:
    """Add empty padding to the provided DataFrame.

    Parameters
    --------------------------
    csv: pd.DataFrame,
        The DataFrame to add the empty padding to.
    state: Random,
        The random state to use for the generation.
    """
    csv = add_row(csv, csv.columns, 0)
    w = len(csv.columns)
    h = len(csv)
    columns_number = state.randint(w, w * 2)
    rows_number = state.randint(h, h * 2)
    empties = np.empty((rows_number, columns_number), dtype="O")
    empties.fill(np.nan)
    if columns_number - w != 0:
        start_x = state.randint(0, columns_number - w)
    else:
        start_x = 0
    if rows_number - h != 0:
        start_y = state.randint(0, rows_number - h)
    else:
        start_y = 0

    empties[start_y : start_y + h, start_x : start_x + w] = csv.values

    return pd.DataFrame(empties, columns=list(range(empties.shape[1])))



"""Submodule to add NaN-like artefacts to a csv file."""

from random import Random
import pandas as pd
import numpy as np
from ugly_csv_generator.utils.add_random_spaces import random_space
from ugly_csv_generator.utils.replace_zeros import (
    ZERO_LOOKING_CHARACTERS,
    UNICODE_ZERO_LOOKING_CHARACTERS,
)

NAN_LIKE_ARTIFACTS = [
    "",
    "\n",
    "\r",
    "\n\r",
    "?",
    "NaN",
    "Nan",
    "_",
    "/",
    " ",
    "-",
    "NA",
    ".",
    np.nan,
    None,
    "null",
    "#RIF!",
    "#N/D",
    "N/A",
] + ZERO_LOOKING_CHARACTERS

UNICODE_NAN_LIKE_ARTIFACTS = [
    "\uFFFD",  # REPLACEMENT CHARACTER
    "\u221E",  # INFINITY
    "\u29DC",  # INCOMPLETE INFINITY
    "\u003F",  # QUESTION MARK
    "\u00BF",  # INVERTED QUESTION MARK
    "\u2216",  # SET MINUS
    "\u65E0",  # CJK UNIFIED IDEOGRAPH representing "none"
] + UNICODE_ZERO_LOOKING_CHARACTERS


def random_nan(state: Random, include_unicode: bool) -> str:
    """Generate a random NaN-like artefact.

    Parameters
    --------------------------
    state: Random,
        The random state to use for the generation.
    """
    if state.randint(0, 1) == 1:
        return state.choice(
            (UNICODE_NAN_LIKE_ARTIFACTS + NAN_LIKE_ARTIFACTS)
            if include_unicode
            else NAN_LIKE_ARTIFACTS
        )
    n = state.randint(1, 10)
    return (
        state.choice([".", "-", "_", "/", random_space(state, include_unicode=False)])
        * n
    )


def add_nan_like_artefacts(
    csv: pd.DataFrame, state: Random, include_unicode: bool
) -> pd.DataFrame:
    """Add NaN-like artefacts to the provided DataFrame.

    Parameters
    --------------------------
    csv: pd.DataFrame,
        The DataFrame to add the NaN-like artefacts to.
    state: Random,
        The random state to use for the generation.
    include_unicode: bool,
        Whether to include Unicode NaN-like artefacts.
    """
    return csv.map(
        lambda x: (
            random_nan(state, include_unicode=include_unicode) if pd.isna(x) else x
        )
    )



"""This module contains the function to duplicate the schema into random new rows."""

from random import Random
import pandas as pd
from tqdm.auto import trange
from ugly_csv_generator.utils.add_row import add_row


def add_duplicate_schema(
    csv: pd.DataFrame, state: Random, verbose: bool
) -> pd.DataFrame:
    """Duplicate the schema into random new rows.

    Parameters
    --------------------------
    csv: pd.DataFrame,
        The DataFrame to add the duplicated schema to.
    state: Random,
        The random state to use for the generation.
    verbose: bool,
        Whether to show the loading bars.
    """
    schema_number = state.randint(0, max(1, len(csv) // 2))
    for _ in trange(
        schema_number,
        desc="Adding duplicated schemas",
        disable=not verbose or schema_number <= 1,
        leave=False,
        dynamic_ncols=True,
    ):
        csv = add_row(csv, csv.columns, state=state)
    return csv



"""Setup for the ugly-csv-generator python package."""

import os
import re
from setuptools import find_packages, setup

here = os.path.abspath(os.path.dirname(__file__))

# Get the long description from the relevant file
with open(os.path.join(here, "README.md"), encoding="utf8") as f:
    long_description = f.read()


def read(*parts):
    with open(os.path.join(here, *parts), "r", encoding="utf8") as fp:
        return fp.read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


__version__ = find_version("ugly_csv_generator", "__version__.py")

test_deps = [
    "pytest",
    "pytest-cov",
    "validate_version_code",
    "random_csv_generator>=1.0.2",
    "pytest-readme",
]

extras = {
    "test": test_deps,
}

setup(
    name="ugly_csv_generator",
    version=__version__,
    description="Python package to generate ugly real-looking csvs.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/LucaCappelletti94/ugly_csv_generator",
    author="LucaCappelletti94",
    author_email="cappelletti.luca94@gmail.com",
    # Choose your license
    license="MIT",
    include_package_data=True,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
    ],
    packages=find_packages(exclude=["contrib", "docs", "tests*"]),
    tests_require=test_deps,
    # Add here the package dependencies
    install_requires=["pandas>=2.1.0", "numpy", "tqdm"],
    python_requires=">=3.9",
    extras_require=extras,
)



"""Test that the format of the version is correct."""

from validate_version_code import validate_version_code
from ugly_csv_generator.__version__ import __version__


def test_version():
    """Test that the format of the version is correct."""
    assert validate_version_code(__version__)



"""Submodule to add random empty rows to a csv file"""

from random import Random
import pandas as pd
import numpy as np
from tqdm.auto import trange
from ugly_csv_generator.utils.add_row import add_row


def add_empty_rows(csv: pd.DataFrame, state: Random, verbose: bool) -> pd.DataFrame:
    """Add random empty rows.

    Parameters
    --------------------------
    csv: pd.DataFrame,
        The DataFrame to add the empty rows to.
    state: Random,
        The random state to use for the generation.
    verbose: bool,
        Whether to show the loading bars.
    """
    rows_number = state.randint(0, max(1, len(csv) // 2))
    empties = np.empty(len(csv.columns))
    empties.fill(np.nan)
    for _ in trange(
        rows_number,
        desc="Adding empty rows",
        disable=not verbose or rows_number <= 1,
        leave=False,
        dynamic_ncols=True,
    ):
        csv = add_row(csv, empties, state=state)
    return csv



"""Submodule to add satellites to a CSV file

A satellite is a small table-like artefact that is added randomly around the CSV
to simulate the presence of random scribbling around the data. While it really should
not exist in well-formed CSV files, it is sadly common in the real world.

A memorable example of a satellite, is when some data-entry person wrote the
order of pizzas for a party on the same sheet as the financial report.
"""

import os
from random import Random
from glob import glob
import pandas as pd
from ugly_csv_generator.utils.add_chunk import add_chunk


def random_satellite(state: Random):
    """Pick a random satellite from the available ones.

    Parameters
    --------------------------
    state: Random,
        The random state to use for the generation.
    """
    satellites = glob(f"{os.path.dirname(os.path.abspath(__file__))}/satellites/*.csv")
    return pd.read_csv(state.choice(satellites), header=None)


def add_satellites(csv: pd.DataFrame, state: Random) -> pd.DataFrame:
    """Add satellites to the provided DataFrame.

    Parameters
    --------------------------
    csv: pd.DataFrame,
        The DataFrame to add the satellites to.
    state: Random,
        The random state to use for the generation.
    """
    if state.randint(0, 1) == 1:
        csv = add_chunk(csv, random_satellite(state), 0)
    if state.randint(0, 1) == 1:
        csv = add_chunk(csv, random_satellite(state), len(csv))
    return csv
